unit uHID;

interface

Uses
  Windows, Classes, SysUtils, Messages,
  JvHidControllerClass,
  HID, HidUsage,
  Dialogs;

type
  TIRData = packed record
    Protocol : Byte;
    Address : Word;
    Command : Word;
    Flags : Byte;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportIRCode = packed record
    ReportID: Byte;
    Data: TIRData;
  end;

  // kein HID-Report kann groesser als 64 Byte sein
  THidIrmpVersion = packed record
    ReportID: Byte;
    Data: Array[0..10] of AnsiChar;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportIRPollingTime = packed record
    ReportID: Byte;
    Data: Word;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportPowerOn = packed record
    ReportID: Byte;
    Data: Boolean;
  end;


type
  THIDThread = class(TThread)
    private
      JvHidDeviceController: TJvHidDeviceController;
      Device: TJvHidDevice;
      IRReport: THidReportIRCode;
      PowerOnReport : THidReportPowerOn;
      ReportIRPollingTime : THidReportIRPollingTime;
      IrmpVersion : THidIrmpVersion;
      PowerOnEnabled : Boolean;
      Last_IR_Buffer: TIRData;
      Msg: TMsg;
      RepeatCounter : Integer;

      USBPoolingDelay : Integer;
      HidVendorID : Word;
      HidProductID : Word;

      //irmp const:
      const
        IRMP_SIRCS_PROTOCOL       = 1;
        IRMP_NEC_PROTOCOL         = 2;
        IRMP_SAMSUNG_PROTOCOL     = 3;
        IRMP_MATSUSHITA_PROTOCOL  = 4;
        IRMP_KASEIKYO_PROTOCOL    = 5;
        IRMP_RECS80_PROTOCOL      = 6;
        IRMP_RC5_PROTOCOL         = 7;
        IRMP_DENON_PROTOCOL       = 8;
        IRMP_RC6_PROTOCOL         = 9;
        IRMP_SAMSUNG32_PROTOCOL   = 10;
        IRMP_APPLE_PROTOCOL       = 11;
        IRMP_RECS80EXT_PROTOCOL   = 12;
        IRMP_NUBERT_PROTOCOL      = 13;

        IRMP_FLAG_REPETITION      = 1;

    protected
      procedure JvHidDeviceArrival(HidDev: TJvHidDevice);
      procedure JvHidDeviceUnplug(HidDev: TJvHidDevice);
      procedure Execute; override;

      //sync with main form:
      procedure SyncIRData;
      procedure SyncDeviceStatus;
      procedure SyncTrainedIRData;
      procedure SyncPowerOnEnabled;
    public
      constructor Create;
      destructor Destroy; override;
      property PoolingDelay: Integer read USBPoolingDelay write USBPoolingDelay;
      property VendorID: Word read HidVendorID write HidVendorID;
      property ProductID: Word read HidProductID write HidProductID;
  end;

const
  MessageToClientThread     =   WM_USER + $518;

  ReadActualIRCode = 1;
  ReadPowerOnEnabled = 2;
  ReadTrainendIRCode = 3;
  SetPowerOnEnabled = 4;
  SetTrainedIRCode = 5;
  SetIRPollingTime = 6;
  ReadIrmpVersion = 7;

var
  HIDThread : THIDThread;
  HIDThreadID : Thandle;


implementation

uses
  uMain;

{
Snync functions
}

procedure THIDThread.SyncPowerOnEnabled;
begin
  if PowerOnEnabled then
    FormMain.Memo1.Lines.Add('PowerOn is Enabled')
  else
    FormMain.Memo1.Lines.Add('PowerOn is Disabled');

  FormMain.Memo1.Lines.Add('');
end;

procedure THIDThread.SyncTrainedIRData;
begin
  FormMain.Memo1.Lines.Add('Trained IR Code:');
  case Last_IR_Buffer.Protocol of
    IRMP_SIRCS_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_SIRCS_PROTOCOL');
    IRMP_NEC_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_NEC_PROTOCOL');
    IRMP_SAMSUNG_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_SAMSUNG_PROTOCOL');
    IRMP_MATSUSHITA_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_MATSUSHITA_PROTOCOL');
    IRMP_KASEIKYO_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_KASEIKYO_PROTOCOL');
    IRMP_RECS80_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RECS80_PROTOCOL');
    IRMP_RC5_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RC5_PROTOCOL');
    IRMP_DENON_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_DENON_PROTOCOL');
    IRMP_RC6_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RC6_PROTOCOL');
    IRMP_SAMSUNG32_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_SAMSUNG32_PROTOCOL');
    IRMP_APPLE_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_APPLE_PROTOCOL');
    IRMP_RECS80EXT_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RECS80EXT_PROTOCOL');
    IRMP_NUBERT_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_NUBERT_PROTOCOL');
  end;

  FormMain.Memo1.Lines.Add('Adress: 0x' + IntToHex(Last_IR_Buffer.Address, 4));
  FormMain.Memo1.Lines.Add('Command: 0x' + IntToHex(Last_IR_Buffer.Command, 4));
  FormMain.Memo1.Lines.Add('Flags: 0x' + IntToHex(Last_IR_Buffer.Flags, 2));
  FormMain.Memo1.Lines.Add('');
end;

procedure THIDThread.SyncIRData;
begin
  FormMain.Memo1.Lines.Add('IR Code received:');
  case Last_IR_Buffer.Protocol of
    IRMP_SIRCS_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_SIRCS_PROTOCOL');
    IRMP_NEC_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_NEC_PROTOCOL');
    IRMP_SAMSUNG_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_SAMSUNG_PROTOCOL');
    IRMP_MATSUSHITA_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_MATSUSHITA_PROTOCOL');
    IRMP_KASEIKYO_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_KASEIKYO_PROTOCOL');
    IRMP_RECS80_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RECS80_PROTOCOL');
    IRMP_RC5_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RC5_PROTOCOL');
    IRMP_DENON_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_DENON_PROTOCOL');
    IRMP_RC6_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RC6_PROTOCOL');
    IRMP_SAMSUNG32_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_SAMSUNG32_PROTOCOL');
    IRMP_APPLE_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_APPLE_PROTOCOL');
    IRMP_RECS80EXT_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_RECS80EXT_PROTOCOL');
    IRMP_NUBERT_PROTOCOL : FormMain.Memo1.Lines.Add('IRMP_NUBERT_PROTOCOL');
  end;

  FormMain.Memo1.Lines.Add('Adress: 0x' + IntToHex(Last_IR_Buffer.Address, 4));
  FormMain.Memo1.Lines.Add('Command: 0x' + IntToHex(Last_IR_Buffer.Command, 4));
  FormMain.Memo1.Lines.Add('Flags: 0x' + IntToHex(Last_IR_Buffer.Flags, 2));
  FormMain.Memo1.Lines.Add('');
end;

procedure THIDThread.SyncDeviceStatus;
begin

  if Assigned(Device) then
  begin
    FormMain.lbDeviceStatus.Caption := 'Device: connected';
    FormMain.lbIrmpVersion.Caption :=
        'Irmp Version: '
        + PAnsiChar(StrPas(IrmpVersion.Data));
  end
  else begin
    FormMain.lbDeviceStatus.Caption := 'Device: disconnected';
    FormMain.lbIrmpVersion.Caption := 'Irmp Version: unknown';
  end;

end;

constructor THIDThread.Create;
begin
  inherited create(True); // CreateSuspended = true
  FreeOnTerminate := False;

  //create HID Handler:
  JvHidDeviceController := TJvHidDeviceController.Create(nil);

  JvHidDeviceController.OnArrival := JvHidDeviceArrival;
  JvHidDeviceController.OnDeviceUnplug := JvHidDeviceUnplug;
end;

destructor THIDThread.Destroy;
begin
  if Assigned(JvHidDeviceController) then
    FreeAndNil(JvHidDeviceController);

  if Assigned(Device) then
    FreeAndNil(Device);

  // Always call the parent destructor after running your own code
  inherited;
end;

procedure THIDThread.JvHidDeviceArrival(HidDev: TJvHidDevice);
begin

  // schauen ob wir nach einem Geraet suchen muessen
  if not Assigned(Device) then
    // schauen ob ein Griffin PowerMate vorhanden ist
    if JvHidDeviceController.CheckOutByID(Device, HidVendorID, HidProductID) then
    begin
      // es wurde ein Geraet gefunden
      // hier gehoeren Initialisierungen des Geraets und der Oberflaeche hin
      // wir machen die LED an
      FillChar(IRReport, SizeOf(IRReport), 0);
      // das ist ziemlich aufwaendig dafuer das wir nur eine Usage im Report setzen wollen
      IRReport.ReportID := ReadActualIRCode;
      // wir wollen in einen Output-Report schreiben
      Device.ReportTypeParam := HidP_Feature;
      // das benennt die spezifische Usage die gesetzt werden soll
      Device.UsagePageParam := HID_USAGE_PAGE_GENERIC;
      Device.LinkCollectionParam := $01; // CollectionType=Application
      Device.UsageParam := HID_USAGE_CONSUMER_UNDEFINED;
      // einmal den letzten empfangen Code lesen um den AVR Status zu setzen
      Device.GetFeature(IRReport, SizeOf(IRReport));
      //read Irmp Version Date:
      FillChar(IrmpVersion, SizeOf(IrmpVersion), 0);
      repeat
        IrmpVersion.ReportID := ReadIrmpVersion;
      until (Device.GetFeature(IrmpVersion, SizeOf(IrmpVersion)) And (IrmpVersion.ReportID = ReadIrmpVersion));
      Synchronize(SyncDeviceStatus);
    end;
end;

procedure THIDThread.JvHidDeviceUnplug(HidDev: TJvHidDevice);
begin
  //device got removed, check if already connected
  if Assigned(Device) then
  begin
    //connected, goit our device removed?
    if Device = HidDev then
    begin
      FreeAndNil(Device);
      //set label status:
      Synchronize(SyncDeviceStatus);
    end;
  end;
end;

procedure THIDThread.Execute;
begin
  //endles loop:
  while not Terminated do
  begin
    //if device is connected:
    if Assigned(Device) then
    begin

        //clear Reports:
        FillChar(IRReport, SizeOf(IRReport), 0);
        FillChar(PowerOnReport, SizeOf(PowerOnReport), 0);
        FillChar(IrmpVersion, SizeOf(IrmpVersion), 0);

        //check first if message got received from mainform:
        PeekMessage(Msg, 0, 0, 0,PM_REMOVE);

        case Msg.message of
          MessageToClientThread :begin
            case Msg.lParam of

              ReadPowerOnEnabled : begin
                repeat
                  PowerOnReport.ReportID := ReadPowerOnEnabled;
                until (Device.GetFeature(PowerOnReport,
                                          SizeOf(PowerOnReport))
                                          And
                                          (PowerOnReport.ReportID
                                          = ReadPowerOnEnabled));
                PowerOnEnabled := PowerOnReport.Data;
                Synchronize(SyncPowerOnEnabled);
              end; //ReadPowerOnEnabled : begin

              SetPowerOnEnabled : begin
                repeat
                  PowerOnReport.ReportID := SetPowerOnEnabled;
                  PowerOnReport.Data := Boolean(Msg.wParam);
                until (Device.SetFeature(PowerOnReport,
                                          SizeOf(PowerOnReport))
                                          And
                                          (PowerOnReport.ReportID
                                          = SetPowerOnEnabled));
                PowerOnEnabled := PowerOnReport.Data;
              end; //SetPowerOnEnabled : begin

              ReadTrainendIRCode : begin
                repeat
                  IRReport.ReportID := ReadTrainendIRCode;
                until (Device.GetFeature(IRReport,
                                          SizeOf(IRReport))
                                          And
                                          (IRReport.ReportID
                                          = ReadTrainendIRCode));
                Last_IR_Buffer := IRReport.Data;
                Synchronize(SyncTrainedIRData);
              end; //ReadTrainendIRCode : begin

              SetTrainedIRCode : begin
                repeat
                  IRReport.ReportID := SetTrainedIRCode;
                until (Device.SetFeature(IRReport,
                                          SizeOf(IRReport))
                                          And
                                          (IRReport.ReportID
                                          = SetTrainedIRCode));
              end; //SetTrainedIRCode : begin

              SetIRPollingTime : begin
                repeat
                  ReportIRPollingTime.ReportID := SetIRPollingTime;
                  ReportIRPollingTime.Data := Word(Msg.wParam);
                until (Device.SetFeature(ReportIRPollingTime,
                                          SizeOf(ReportIRPollingTime))
                                          And
                                          (ReportIRPollingTime.ReportID
                                          = SetIRPollingTime));
              end; //SetIRPollingTime : begin

              ReadIrmpVersion : begin
                repeat
                  IrmpVersion.ReportID := ReadIrmpVersion;
                until (Device.GetFeature(IrmpVersion,
                                          SizeOf(IrmpVersion))
                                          And
                                          (IrmpVersion.ReportID
                                          = ReadIrmpVersion));
                Synchronize(SyncDeviceStatus);
              end; //ReadIrmpVersion : begin
            end; //case Msg.lParam of
          end; //MessageToClientThread :begin
        end; // case Msg.message of

        //clear message:
        Msg.message := 0;
        Msg.lParam := 0;
        Msg.wParam := 0;

        //clear Reports:
        FillChar(IRReport, SizeOf(IRReport), 0);

        // do pooling of device:
        IRReport.ReportID := ReadActualIRCode;
        Device.GetFeature(IRReport, SizeOf(IRReport));

        //check if new data is aviable
        if (IRReport.Data.Protocol <> 0) Then
        begin
          if ((IRReport.Data.Flags And IRMP_FLAG_REPETITION) = IRMP_FLAG_REPETITION) And
            (RepeatCounter = 0) then
            Inc(RepeatCounter)
          else
          if ((IRReport.Data.Flags And IRMP_FLAG_REPETITION) = IRMP_FLAG_REPETITION) And
            (RepeatCounter > 0) then
          begin
            Inc(RepeatCounter);
            if RepeatCounter >= 5 then
            begin
              Last_IR_Buffer := IRReport.Data;
              Synchronize(SyncIRData);
            end;
          end
          else begin
            RepeatCounter := 0;
            Last_IR_Buffer := IRReport.Data;
            Synchronize(SyncIRData);
          end;
        end; //if (IR_Buffer.IRData.Protocol <> 0) Then
    end; //if Assigned(Device) then
    //do USB polling time delay
    Sleep(USBPoolingDelay);
  end; //while not Terminated do
end;

end.
